Attribute VB_Name = "modMain"
'****************************************************************
'   This is an example program to show you how to use           '
'   mod3Denhance.bas, which is a module that allows             '
'   you to use Direct3D for bltting to the screen.              '
'   This is very useful for some features that are not          '
'   available in DDraw, eg. Alpha-Blending.                     '
'   It is also useful for isometrics, as your not restricted    '
'   to rectangles.                                              '
'                                                               '
'   Feel free to use this code for anything you want, but       '
'   if you do use it for anything, then I'd appreciate an       '
'   E-mail telling me what you've done and a Thank-you          '
'   would be nice aswell.                                       '
'   Also, please E-mail me with any comments, suggestions or    '
'   problems with the code.                                     '
'   Or Please just E-mail to say what you think of the code     '
'                                                               '
'   Carl Warwick     - ( carlwarwick@strayduck.com )            '
'   Freeride Designs - ( http://freeridedesign.cjb.net )        '
'   13 - May - 2000                                             '
'****************************************************************

Option Explicit

Private Sub Main()
    frmMain.Show
    
    'Initialize DirectX with screen 640x480x16
    Call Initialize_DX(frmMain, 640, 480, 16)
    
    'Create our two texture surfaces
    'Note...
    'Texture surface dimensions must be to a power or 2 (^2)
    'eg. possible sizes are :- 256x256, 128x128, 128,256, 16x16 etc...
    'Its not advisable to use surface bigger than 256x256 or at the most 512x512
    Set ddsTexture1 = CreateTextureSurface(App.Path & "\tex1.bmp", 128, 128, 2)
    Set ddsTexture2 = CreateTextureSurface(App.Path & "\tex2.bmp", 128, 256, 0)
    
    'Create a normal DDraw surface
    Call DDCreateSurface(ddsurface, 640, 480, App.Path & "\BG.bmp", 0, False)
    
    'Sets the forecolor, this is only to make the text white when displaying
    'the Frames per second
    Call BackBuffer.SetForeColor(RGB(255, 255, 255))
    
    'Start the main loop
    bRunning = True
    Call Main_Loop
    
    'End the program and return to your normal life
    Endit
End Sub


'The main game loop is in here
Public Sub Main_Loop()
    Dim mlngTimer As Long           '----}
    Dim mlngFrameTimer As Long      '    }  Used to calculate the Frames per second
    Dim mintFPSCounter As Integer   '----}
    Dim FPS As Integer              'Holds the number of frames per second


    Dim tmpRect As RECT             'A RECT to be used for bltting
    tmpRect.Left = 0: tmpRect.Top = 0
    tmpRect.Right = 640: tmpRect.Bottom = 480
    
    
    Dim x As Single                 'Used for the position of the foreground
    Dim step As Integer             'Which direction is the foreground moving
    x = 0: step = 1                 'initialize the variables
    
    
    'Position our vertex list for the Freeride Designs logo
    'We don't want to set any corner colours
    Call Set_Vertex(logoVertex(), 500, 332, 628, 332, 628, 460, 500, 460)
    
    
    'Start our loop, press the 'Esc' key to exit
    Do Until bRunning = False
        
        'Move the foregrounds positions
        If x >= 320 Then step = -1
        If x <= 0 Then step = 1
        x = x + step
        
        
        'Position our foreground vertex list.
        'Use DX.CreateColorRGB to define colours for each corner
        Call Set_Vertex(foreVertex(), x, 30, 640 - x, 30, 640 - x, 450, x, 450, DX.CreateColorRGB(0, 0, 0), DX.CreateColorRGB(255, 255, 0), DX.CreateColorRGB(255, 255, 255), DX.CreateColorRGB(128, 0, 0))
    
    
        'Clear the device after all the calls to Set_Vertex, and before
        'calling Device.BeginScene and any bltting.
        Call Clear_Device
        
        
        'Use Bltfast on our normal DDraw surface to draw the background
        Call BackBuffer.BltFast(0, 0, ddsurface, tmpRect, DDBLTFAST_WAIT)
        
        
        'Begin the scene, must do this after Calling Clear_Device, and before
        'calling Blt3D()
        Device.BeginScene
        
            'We don't want to show the magneta color on our logo surface,
            'so call Set_ColorKey True to enable color keying
            '
            'Turning on/off Color keying lots of times in one loop can
            'be slow, so make sure you do all your Color keyed bltting
            'at the same time, so Set_ColorKey is only called once per loop
            'to turn on, and oncec per loop to turn off.
            Set_ColorKey True
            'Blt our logo texture surface at the positions defined by logoVertex()
            'Which was set earlier by Set_Vertex()
            Call Blt3D(ddsTexture1, logoVertex())
            'Disable colour keying from future blting operations
            Set_ColorKey False
            
            
            'We want our foreground texture surface to be alpha blended,
            'so call Set_Alphablend True to enable Alpha blending.
            '
            'Turning on/off alphablending lots of times in one loop can
            'be slow, so make sure you do all your alpha blended bltting
            'at the same time, so Set_Alphablend is only called once per loop
            'to turn on, and oncec per loop to turn off.
            Set_Alphablend True
            'Blt our foreground texture surface at the positions defined by
            'logoVertex(), which was set earlier by Set_Vertex()
            Call Blt3D(ddsTexture2, foreVertex())
            'Disable Alpha blending from future blting operatons
            Set_Alphablend False
            
            
        'Call EndScene when finished using D3D routines
        Device.EndScene
            
        
        '===============================
        'Calculate the frame rate
        '(Thanks to lucky for this code)
        '===============================
        mlngTimer = DX.TickCount()
        If DX.TickCount() - mlngFrameTimer >= 1000 Then
            mlngFrameTimer = DX.TickCount()
            FPS = mintFPSCounter
            mintFPSCounter = 0
        Else
            mintFPSCounter = mintFPSCounter + 1
        End If
        'Display the FPS
        Call BackBuffer.DrawText(0, 0, "FPS :" & Format(FPS), False)
        
        
        'Flip the primary surface
        Primary.Flip Nothing, DDFLIP_WAIT
        
        'Let windows do its stuff
        DoEvents
    Loop
End Sub
